@@ -21,14 +21,22 @@ mod macros;
 
 /// Types and traits implemented by backend [formatters](backend::Format)
 pub mod backend;
+
 /// Helper modules for implementing buffered serialization primitives
 pub mod buffer;
-/// Helper modules that may be externally useful
-pub mod util;
+
+/// Types used to support deserialization streams
+pub mod deserializer;
 
 /// Re-export of io related structures
 pub mod io;
 
+/// Types used to suport serialization sinks
+pub mod serializer;
+
+/// Helper modules that may be externally useful
+pub mod util;
+
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 #[cfg(feature = "diny_derive")]
 pub use diny_derive::AsyncSerialization;
@@ -46,4 +54,7 @@ pub use backend::{
     AsyncDeserialize,
     AsyncSerialize,
     AsyncSerialization,
-};
\ No newline at end of file
+};
+
+pub use deserializer::{deserializer, Deserialize, Deserializer};
+pub use serializer::{serializer, Serialize, Serializer};
